<?php

namespace App\Helpers;

use App\User;
use Illuminate\Support\Facades\Http;

class Requester
{

    private $localHandler;
    private $request_url;
    private $secret_key;

    function __construct()
    {
        $this->request_url = env('PRODUCTION');
        $this->secret_key = env('SECRET_KEY');
        $this->localHandler = new LocalHandler();
    }


    /**
     * fungsi generate jwt untuk inisialisasi session API
     * @param string $emailId
     * @return string Token
     */
    private function calculateSessionId(string $emailId) : string {

        $arrHeader = ['typ' => 'JWT', 'alg' => 'HS256'];
        $header = base64_encode(json_encode($arrHeader));
        $header = str_replace(['+', '/'], ['-', '_'] ,$header);

        $secret = $this->secret_key;
        $issuedAt = time(); /*epochtime / unix timestamp*/
        $expired = $issuedAt + 1800; /*expired dalam 30 menit*/
        $arrPayload = [
            "email" => $emailId,
            "iat"   => $issuedAt,
            "exp"   => $expired
        ];
        $payload = base64_encode(json_encode($arrPayload));
        $payload = str_replace(['+', '/'], ['-', '_'] ,$payload);

        $signature = base64_encode(hash_hmac('sha256',$header.'.'.$payload,$secret,true));
        $jwt = $header.'.'.$payload.'.'.$signature;
        $jwt = str_replace(['+', '/'], ['-', '_'] ,$jwt);

        return $jwt;
    }

    /**
     * fungsi login
     * @param string $userEmail
     * @param string $password
     * @return ResponseFormat
     */
    private function real_login(string $userEmail, 
                                string $password,
                                string $sessionID) : array
    {

        $request = [
            "method" => 'getloginweb',
            "email" => $userEmail,
            "pass" => $password,
            "sessionID" => $sessionID,
        ];

        $httpResponse = Http::contentType('application/json')
            ->post($this->request_url, $request);
        $json = $httpResponse->json();

        return $json;
    }



    /**
     * fungsi login
     * @param string $userEmail
     * @param string $password
     * @return ResponseFormat
     */
    public function login (string $userEmail, string $password) : ResponseFormat
    {
            $sessionID = $this->calculateSessionId($userEmail);
            $json = $this->real_login($userEmail, $password, $sessionID);
            $status = $json["status"];
            $message = $json["message"];
            $displayMessage = $json["displayMessage"] ?? $message;
            $accessToken = $json["accessToken"];
            $frc_aktif = $json["frc_aktif"]??'N';
            $last_login = $json["last_login"]??NULL;
            $last_update = $json["last_update"]?? NULL;
            $responseFormat = new ResponseFormat($status, $message, $displayMessage);
            $responseFormat->setPayload([
                "accessToken" => $accessToken,
                "sessionId"   => $sessionID,
                "franchisee"  => $frc_aktif,
                "last_login"  => $last_login,
                "last_update" => $last_update
            ]);

            return $responseFormat;
    }


    private function real_post(string $method, array $params, User $user) {
        $httpResponse = Http::contentType('application/json')
            ->post($this->request_url, $params);

        $json = $httpResponse->json();
        if ($method == 'logout') {
            $user->delete();  //delete session
            return ["status" => 'T', "message" => "Success"];
        }

        $accessToken = $json["accessToken"];
        $user->apiAccessToken = $accessToken;
        $user->save();
        return $json;
    }

    private $reserved_fields = ["status", "message", "displayMessage", "accessToken", "sessionId"];


    /**
     * fungsi form
     * @param string $userEmail
     * @param string $password
     * @return ResponseFormat
     */
    public function form(string $method, array $params, User $user)
    {
        if (!isset($method)) {
            return ResponseFormat::notImpl();
        }
        if ($this->localHandler->has_handled_method($method)){

            $json = $this->localHandler->handle_post($method, $params, $user);
            $status = $json["status"];
            $message = $json["message"];
            $displayMessage = $json["displayMessage"] ?? '';
            $responseFormat = new ResponseFormat($status, $message, $displayMessage);
            $responseFormat->setPayload($json["payload"]);
            return $responseFormat;
        }
        else {
            $params['email'] = $user->email;
            $params['accessToken'] = $user->apiAccessToken;
            $params['sessionId'] = $user->apiSessionId;

            $json = $this->real_post($method, $params, $user);

            $status = $json["status"];
            $message = $json["message"];
            $displayMessage = $json["displayMessage"] ?? '';
            $accessToken = $json["accessToken"];
            $user->apiAccessToken = $accessToken;
            $user->save();

            $responseFormat = new ResponseFormat($status, $message, $displayMessage);
            $payload = [];
            foreach ($json as $key => $value) {
                if (!in_array($key, $this->reserved_fields)) {
                    $payload[$key] = $value;
                }
            }
            $responseFormat->setPayload($payload);

            return $responseFormat;
        }

        //return ResponseFormat::notImpl('login');
    }    

    private function real_post_public(string $method, array $params) {

        $httpResponse = Http::contentType('application/json')
            ->post($this->request_url, $params);

        $json = $httpResponse->json();
       return $json;
    }

    public function formPublic(string $method, array $params){
        // check for allowed methods
        if (in_array($method, ['lostPass',  'resetPass', 'konfirmasiEmail', 'mailActivation'])){
            // if ($method != 'reregistration') {
            $params['sessionID'] = $this->calculateSessionId($params['email']);
            // }
            $json = $this->real_post_public($method, $params);

            $status = $json["status"];
            $message = $json["message"];
            $displayMessage = $json["displayMessage"] ?? '';
            $responseFormat = new ResponseFormat($status, $message, $displayMessage);
            $payload = [];
            foreach ($json as $key => $value) {
                if (!in_array($key, $this->reserved_fields)) {
                    $payload[$key] = $value;
                }
            }
            $responseFormat->setPayload($payload);

            return $responseFormat;

        }
        else {
            return ResponseFormat::notImpl($method);
        }        
    }

    public function download(string $method, array $params, User $user) {
        // TODO download of files stored on the gateway
        // 
    }    



}
